#!/bin/bash

DEFAULT_PATHS=(/var/abs/core /var/abs/extra)

parse() {
	unset pkgname pkgver pkgrel pkgdesc url license groups provides md5sums
	unset replaces depends conflicts backup source install build makedepends
	unset optdepends options noextract
	ret=0
	source $1 &>/dev/null || ret=$?

	#if [ $ret -ne 0 ]; then
	#	echo -e "brokenpackages.append(\"$1\")"
	#	return
	#fi

	# ensure $pkgname and $pkgver variables were found
	if [ $ret -ne 0 -o -z "$pkgname" -o -z "$pkgver" ]; then
		echo -e "invalid_pkgbuilds.append(\"$1\")"
		return
	fi 

	echo -e "pkg = PacmanPackage()"
	echo -e "pkg.name = \"$pkgname\""
	echo -e "packages[pkg.name] = pkg"
	echo -e "pkg.version = \"$pkgver-$pkgrel\""
	echo -e "pkg.path = \"$1\""

	if [ -n "$arch" ]; then
		for i in ${arch[@]}; do
			echo -e "pkg.archs.append(\"$i\")"
		done
	fi
	if [ -n "$depends" ]; then
		for i in ${depends[@]}; do
			echo -e "pkg.deps.append(\"$i\")"
		done
	fi
	if [ -n "$makedepends" ]; then
		for i in ${makedepends[@]}; do
			echo -e "pkg.makedeps.append(\"$i\")"
		done
	fi
	if [ -n "$conflicts" ]; then
		for i in ${conflicts[@]}; do
			echo -e "pkg.conflicts.append(\"$i\")"
		done
	fi
	if [ -n "$provides" ]; then
		for i in ${provides[@]}; do
			echo -e "pkg.provides.append(\"$i\")"
			prov="\"${i/=*/}\""
			echo -e "if not provisions.has_key($prov):"
			echo -e "\tprovisions[$prov] = []"
			echo -e "provisions[$prov].append(pkg)"
		done
	fi
}

parse_all() {
	source /etc/makepkg.conf
	export CARCH

	cat << EOF
class PacmanPackage:
	def __init__(self):
		self.name,self.version,self.path = "","",""
		self.deps,self.makedeps,self.provides,self.conflicts,self.archs = [],[],[],[],[]
packages = {}
provisions = {}
invalid_pkgbuilds = []
EOF

	find $@ -name PKGBUILD | while read pkgbuild; do
		parse $pkgbuild
	done
}

PATHS=${*:-${DEFAULT_PATHS[*]}}
parse_all "${PATHS[*]}" > packages.py

# ugly hack to strip the weird klibc- deps
sed -i '/klibc-[A-Za-z0-9]\{20\}/d' packages.py

python check_packages.py

