#!/usr/bin/python

valid_archs = ['i686', 'x86_64']

import os
import alpm
from packages import PacmanPackage,packages,provisions,invalid_pkgbuilds

class Depend:
	def __init__(self,name,version,mod):
		self.name = name
		self.version = version
		self.mod = mod

def splitdep(dep):
	name = dep
	version = ""
	mod = ""
	for char in (">=", "<=", "=", ">", "<"):
		pos = dep.find(char)
		if pos > -1:
			name = dep[:pos]
			version = dep[pos:].replace(char, "")
			mod = char
			break
	return Depend(name,version,mod)

def splitprov(prov):
	name = prov
	version = ""
	pos = prov.find("=")
	if pos > -1:
		name = prov[:pos]
		version = prov[pos:].replace("=", "")
	return (name,version)

def vercmp(v1,mod,v2):
	#compare_str = "vercmp " + v1 + " " + v2
	#res = int(os.popen(compare_str).read().replace("\n", ""))
	res = alpm.vercmp(v1,v2)
	if res == 0:
		return (mod.find("=") > -1)
	elif res < 0:
		return (mod.find("<") > -1)
	elif res > 0:
		return (mod.find(">") > -1)
	return False


def depcmp(name,version,dep):
	if name != dep.name:
		return False
	if dep.version == "" or dep.mod == "":
		return True
	if version == "":
		return False
	#print "depcmp : " + version + dep.mod + dep.version
	return vercmp(version,dep.mod,dep.version)

def provcmp(pkg,dep):
	for prov in pkg.provides:
		(provname,provver) = splitprov(prov)
		if depcmp(provname,provver,dep):
			return True
	return False

def verify_dep(dep):
	dep = splitdep(dep)
	if packages.has_key(dep.name):
		pkg = packages[dep.name]
		if depcmp(pkg.name,pkg.version,dep):
			return True
	if provisions.has_key(dep.name):
		provlist = provisions[dep.name]
		for prov in provlist:
			if provcmp(prov,dep):
				return True
	return False

def verify_deps(name,deps):
	missdeps = []
	for dep in deps:
		if verify_dep(dep) == False:
			missdeps.append(name + " --> '" + dep + "'")
	return missdeps

def verify_archs(name,archs):
	invalid_archs = []
	for arch in archs:
		if arch not in valid_archs:
			invalid_archs.append(name + " --> " + arch)
	return invalid_archs

def print_heading(heading):
	print ""
	print "=" * (len(heading) + 4)
	print "= " + heading + " ="
	print "=" * (len(heading) + 4)

def print_subheading(subheading):
	print ""
	print subheading
	print "-" * (len(subheading) + 2)

def print_missdeps(pkgname,missdeps) :
	for d in missdeps:
		print pkgname + " : " + d

def print_result(list, subheading):
	if len(list) > 0:
		print_subheading(subheading)
		for item in list:
			print item

def print_results():
	print_result(invalid_pkgbuilds, "Invalid PKGBUILDs")
	print_result(mismatches, "Mismatched Pkgnames")
	print_result(invalid_archs, "Invalid Archs")
	print_result(missing_deps, "Missing Dependencies")
	print_result(missing_makedeps, "Missing Makedepends")
	print_subheading("Summary")
	print "Invalid PKGBUILDs:                    ", len(invalid_pkgbuilds)
	print "Mismatching PKGBUILD names:           ", len(mismatches)
	print "Invalid archs:                        ", len(invalid_archs)
	print "Missing (make)dependencies:           ", len(missing_deps)+len(missing_makedeps)
	print ""

mismatches = []
missing_deps = []
missing_makedeps = []
invalid_archs = []
print_heading("Integrity Check")
print "\nPerforming integrity checks..."
print "==> checking mismatches"
for name,pkg in packages.iteritems():
	pkgdirname = pkg.path.split("/")[-2]
	if name != pkgdirname:
		mismatches.append(name + " vs. " + pkgdirname)
print "==> checking archs"
for name,pkg in packages.iteritems():
	archs = verify_archs(name,pkg.archs)
	invalid_archs.extend(archs)
print "==> checking dependencies"
for name,pkg in packages.iteritems():
	missdeps = verify_deps(name,pkg.deps)
	missing_deps.extend(missdeps)
print "==> checking makedepends"
for name,pkg in packages.iteritems():
	missdeps = verify_deps(name,pkg.makedeps)
	missing_makedeps.extend(missdeps)
print_results()
